linkage <- function( freq.A1.pop1=1, freq.B1.pop1=1, 
                     freq.A1.pop2=0, freq.B1.pop2=0,
                     n.pop1=50, n.pop2=50,
                     rho=0.1, stoptime=100 ) 
{
   # check freq.A1.pop1
   if ( freq.A1.pop1 < 0 || freq.A1.pop1 > 1 ) 
      stop( "freq.A1.pop1 must be between 0 and 1" )

   # check freq.B1.pop1
   if ( freq.B1.pop1 < 0 || freq.B1.pop1 > 1 ) 
      stop( "freq.B1.pop1 must be between 0 and 1" )

   # check freq.A1.pop2
   if ( freq.A1.pop2 < 0 || freq.A1.pop2 > 1 ) 
      stop( "freq.A1.pop2 must be between 0 and 1" )

   # check freq.B1.pop2
   if ( freq.B1.pop2 < 0 || freq.B1.pop2 > 1 ) 
      stop( "freq.B1.pop2 must be between 0 and 1" )

   # check n.pop1
   if ( n.pop1 <= 0 ) 
      stop( "n.pop1 must be greater than 0" )

   # check n.pop2
   if ( n.pop2 <= 0 ) 
      stop( "n.pop2 must be greater than 0" )

   # check rho
   if ( rho < 0 || rho > 0.5 ) 
      stop( "rho must be between 0 and 1" )

   # check stoptime
   if ( stoptime <= 0 ) 
      stop( "stoptime must be greater than 0" )

   n <- n.pop1 + n.pop2

   freq.A2.pop1 = 1 - freq.A1.pop1
   freq.B2.pop1 = 1 - freq.B1.pop1
   freq.A2.pop2 = 1 - freq.A1.pop2
   freq.B2.pop2 = 1 - freq.B1.pop2

   # heterozygozities
   h.A <- 2 * ( freq.A1.pop1 * freq.A2.pop1 * n.pop1 +
                freq.A1.pop2 * freq.A2.pop2 * n.pop2 ) / n
   h.B <- 2 * ( freq.B1.pop1 * freq.B2.pop1 * n.pop1 +
                freq.B1.pop2 * freq.B2.pop2 * n.pop2 ) / n

   # genotype frequencies in generation 0
   A1B1 <- ( n.pop1 * freq.A1.pop1 * freq.B1.pop1 +
             n.pop2 * freq.A1.pop2 * freq.B1.pop2 ) / n
   A1B2 <- ( n.pop1 * freq.A1.pop1 * freq.B2.pop1 +
             n.pop2 * freq.A1.pop2 * freq.B2.pop2 ) / n
   A2B1 <- ( n.pop1 * freq.A2.pop1 * freq.B1.pop1 +
             n.pop2 * freq.A2.pop2 * freq.B1.pop2 ) / n
   A2B2 <- ( n.pop1 * freq.A2.pop1 * freq.B2.pop1 +
             n.pop2 * freq.A2.pop2 * freq.B2.pop2 ) / n
   Ht.freq <- data.frame( "A1B1"=A1B1, "A1B2"=A1B2,
                          "A2B1"=A2B1, "A2B2"=A2B2 )

   # allele frequencies in generation 0
   A1 <- A1B1 + A1B2
   A2 <- A2B1 + A2B2
   B1 <- A1B1 + A2B1
   B2 <- A1B2 + A2B2
   A.freq <- data.frame( "A1"=A1, "A2"=A2, "B1"=B1, "B2"=B2 )
   
   for ( i in 1:stoptime ) {

      A1B1 <- ( 1 - rho ) * Ht.freq$A1B1[i] +
              rho * A.freq$A1[i] * A.freq$B1[i]
      A1B2 <- ( 1 - rho ) * Ht.freq$A1B2[i] +
              rho * A.freq$A1[i] * A.freq$B2[i]
      A2B1 <- ( 1 - rho ) * Ht.freq$A2B1[i] +
              rho * A.freq$A2[i] * A.freq$B1[i]
      A2B2 <- ( 1 - rho ) * Ht.freq$A2B2[i] +
              rho * A.freq$A2[i] * A.freq$B2[i]

      new<-rmultinom(1,n,c(A1B1, A1B2, A2B1, A2B2))/n

      A1B1<-new[1,1]
      A1B2<-new[2,1]
      A2B1<-new[3,1]
      A2B2<-new[4,1]

      Ht.freq <- rbind( Ht.freq, c( A1B1, A1B2, A2B1, A2B2 ) )
#      Ht.freq <- rbind( Ht.freq, new[1,] )
      
      A1 <- A1B1 + A1B2
      A2 <- A2B1 + A2B2
      B1 <- A1B1 + A2B1
      B2 <- A1B2 + A2B2

      A.freq <- rbind( A.freq, c( A1, A2, B1, B2 ) )
   }

   # linkage disequilibrium
   D <- Ht.freq$A1B1 * Ht.freq$A2B2 - Ht.freq$A1B2 * Ht.freq$A2B1
   r.squared <- ( D * D ) /
                ( A.freq$A1 * A.freq$A2 * A.freq$B1 * A.freq$B2 )

   # Hardy-Weinberg equilibrium, inbreeding coefficient F = (H0-H)/H0
   H.A <- 2 * ( Ht.freq$A1B1 + Ht.freq$A1B2 ) *
              ( Ht.freq$A2B1 + Ht.freq$A2B2 )
   F.A <- ( H.A[1] - H.A ) / H.A[1]
     
   H.B <- 2 * ( Ht.freq$A1B1 + Ht.freq$A2B1 ) *
              ( Ht.freq$A1B2 + Ht.freq$A2B2 )
   F.B <- ( H.B[1] - H.B ) / H.B[1]

   ret <- list( "Haplotype.freq"=Ht.freq, "Allele.freq"=A.freq,
                "LD"=D, "R.squared"=r.squared,
                "HW.A"=H.A, "F.A"=F.A, "HW.B"=H.B, "F.B"=F.B )

   class( ret ) <- "linkage"

   return( ret )
}

plot.linkage <- function( x, what=c( "A1B1", "A1B2", "A2B1", "A2B2" ), 
                          col=vector(), xlim=NULL, ylim=c( 0, 1 ),
                          xlab="time", ylab="freq", 
                          legend.pos="bottomright", ... )
{
   types <- c( "A1B1", "A1B2", "A2B1", "A2B2", "A1", "A2", "B1", "B2",
               "LD", "R.squared", "HW.A", "F.A", "HW.B", "F.B" )

   stoptime <- nrow( x$Haplotype.freq ) - 1

   legend <- vector()
   legend.col <- vector()
   
   col["A1B1"] <- ifelse( is.na( col["A1B1"] ), "red", col["A1B1"] )
   col["A1B2"] <- ifelse( is.na( col["A1B2"] ), "green", col["A1B2"] )
   col["A2B1"] <- ifelse( is.na( col["A2B1"] ), "blue", col["A2B1"] )
   col["A2B2"] <- ifelse( is.na( col["A2B2"] ), "orange", col["A2B2"] )

   col["A1"] <- ifelse( is.na( col["A1"] ), "pink", col["A1"] )
   col["A2"] <- ifelse( is.na( col["A2"] ), "purple", col["A2"] )
   col["B1"] <- ifelse( is.na( col["B1"] ), "limegreen", col["B1"] )
   col["B2"] <- ifelse( is.na( col["B2"] ), "olivedrab", col["B2"] )

   col["LD"] <- ifelse( is.na( col["LD"] ), "rosybrown", col["LD"] )
   col["R.squared"] <- ifelse( is.na( col["R.squared"] ), "sandybrown", 
                                                          col["R.squared"] )

   col["HW.A"] <- ifelse( is.na( col["HW.A"] ), "skyblue", col["HW.A"] )
   col["F.A"] <- ifelse( is.na( col["F.A"] ), "slateblue", col["F.A"] )
   col["HW.B"] <- ifelse( is.na( col["HW.B"] ), "springgreen", col["HW.B"] )
   col["F.B"] <- ifelse( is.na( col["F.B"] ), "yellowgreen", col["F.B"] )

   if ( is.null( xlim ) ) {
      xlim <- c( 0, stoptime )
   }
   
   plot( 0, 0, xlim=xlim, ylim=ylim, type="n",
         xlab=xlab, ylab=ylab, ... )

   for ( type in c( "A1B1", "A1B2", "A2B1", "A2B2" ) ) {
      if ( type %in% what ) {
         lines( 0:stoptime, x$Haplotype.freq[,type], col=col[type] )
         legend <- c( legend, type )
         legend.col <- c( legend.col, col[type] )
      }
   }

   for ( type in c( "A1", "A2", "B1", "B2" ) ) {
      if ( type %in% what ) {
         lines( 0:stoptime, x$Allele.freq[,type], col=col[type] )
         legend <- c( legend, type )
         legend.col <- c( legend.col, col[type] )
      }
   }

   if ( "LD" %in% what ) {
      lines( 0:stoptime, x$LD, col=col["LD"] )
      legend <- c( legend, "LD" )
      legend.col <- c( legend.col, col["LD"] )
   }

   if ( "R.squared" %in% what ) {
      lines( 0:stoptime, x$R.squared, col=col["R.squared"] )
      legend <- c( legend, "R.squared" )
      legend.col <- c( legend.col, col["R.squared"] )
   }

   if ( "HW.A" %in% what ) {
      lines( 0:stoptime, x$HW.A, col=col["HW.A"] )
      legend <- c( legend, "HW.A" )
      legend.col <- c( legend.col, col["HW.A"] )
   }

   if ( "HW.B" %in% what ) {
      lines( 0:stoptime, x$HW.B, col=col["HW.B"] )
      legend <- c( legend, "HW.B" )
      legend.col <- c( legend.col, col["HW.B"] )
   }

   if ( "F.A" %in% what ) {
      lines( 0:stoptime, x$F.A, col=col["F.A"] )
      legend <- c( legend, "F.A" )
      legend.col <- c( legend.col, col["F.A"] )
   }

   if ( "F.B" %in% what ) {
      lines( 0:stoptime, x$F.B, col=col["F.B"] )
      legend <- c( legend, "F.B" )
      legend.col <- c( legend.col, col["F.B"] )
   }

   legend( legend.pos, legend=legend,
           col=legend.col, lty=1, bg="white" )
}
