popSubdivision <- function( N=50, demes=10, init.A=0.5, m=0, mainland=FALSE,
                            stoptime=500 )
{
   # check N
   if ( N <= 0 )
      stop( "N must be greater than 0" )
   
   # check demes
   if ( demes <= 0 ) 
      stop( "demes must be greater than 0" )

   # check init.A
   if ( init.A < 0 || init.A > 1 ) 
      stop( "init.A must be between 0 and 1" )

   # check m
   if ( m < 0 || m > 1 )
      stop( "m must be between 0 and 1" )

   # check mainland
   if ( ! is.logical( mainland ) )
      stop( "mainland must be either TRUE or FALSE" )

   # check stoptime
   if ( stoptime <= 0 ) 
      stop( "stoptime must be greater than 0" )

   nn <- 2*N
   
   deme.freq.A <- matrix( rep( init.A, demes ), ncol=demes )
   migrant.freq.A <- init.A
   
   Hdeme <- vector()
   Hs <- vector()
   Ht <- vector()
   
   for ( i in 1:stoptime ) {
      # Anzahl der Migranten; pro deme unterschiedlich 
      NumMig <- rbinom( demes, nn, m )
      NumRes <- nn - NumMig
   
      # Anzahl der Migranten mit Allel A
      mig.A <- rbinom( demes, NumMig, migrant.freq.A[i] )
      # Anzahl der Verbleibenden mit Allel A
      res.A <- rbinom( demes, NumRes, deme.freq.A[i,] )
      
      deme.freq.A <- rbind( deme.freq.A, ( res.A + mig.A ) / nn )
      migrant.freq.A <- c( migrant.freq.A,
                           ifelse( mainland, init.A, mean( deme.freq.A[i+1,] ) ) )
   }

   Hdeme <- 2 * deme.freq.A * ( 1 - deme.freq.A )
   Hs <- apply( deme.freq.A, 1, mean )
   Ht <- 2 * migrant.freq.A * ( 1 - migrant.freq.A )
   
   Fst <- ifelse( Ht == 0, 0, ( Ht - Hs ) / Ht )
   Fpredicted <- 1 / ( 1 + 4 * N * m )
   
   fixed <- ifelse( Hdeme == 0, TRUE, FALSE )
   
   prop.fixed <- apply( fixed, 1, sum ) / demes
   
   ret <- list( "deme.freq.A"=deme.freq.A, "migrant.freq.A"=migrant.freq.A,
                "Hdeme"=Hdeme, "Hs"=Hs, "Ht"=Ht, "Fst"=Fst,
                "F.predicted"=Fpredicted, "fixed"=fixed,
                "prop.fixed"=prop.fixed )
   class( ret ) <- "popSubdivision"
   
   return( ret )
}

plot.popSubdivision <- function( x, what=c( "deme.freq.A" ), col=list(),
                                 xlim=NULL, ylim=c( 0, 1 ),
                                 xlab="time", ylab="freq",
                                 legend.pos="bottomright", ... )
{
   stoptime = nrow( x$deme.freq.A ) - 1
   demes = ncol( x$deme.freq.A )

   legend <- vector()
   legend.col <- vector()

   if ( ! "deme.freq.A" %in% names( col ) ) {
      col$deme.freq.A = rainbow( demes )
   }
   if ( ! "migrant.freq.A" %in% names( col ) ) {
      col$migrant.freq.A = "black"
   }

   if ( is.null( xlim ) ) {
      xlim <- c( 0, stoptime )
   }

   plot( 0,0, xlim=xlim, ylim=ylim, type="n",
         xlab=xlab, ylab=ylab, ... )

   if ( "deme.freq.A" %in% what ) {
      for ( i in 1:demes ) {
         lines( 0:stoptime, x$deme.freq.A[,i], col=col$deme.freq.A[i] )
         legend <- c( legend, paste( "deme", i ) )
         legend.col <- c( legend.col, col$deme.freq.A[i] )
      }
   }

   if ( "migrant.freq.A" %in% what ) {
      lines( 0:stoptime, x$migrant.freq.A, col=col$migrant.freq.A )
      legend <- c( legend, "migrant.freq" )
      legend.col <- c( legend.col, col$migrant.freq.A )
   }

   
   legend( legend.pos, legend=legend,
           col=legend.col, lty=1, bg="white" )
}
