seqEvoHitchhiking<-function(N=10, s=0.1, rho=0.01, u=0.1, wait=0, showFixed=FALSE, seq=TRUE, sfs=FALSE, seed=0) {
  if(seed) set.seed(seed)

  # N>1 necessary
  if(N<2) {
    print("error: N>1 necessary")
  }

  # either seq or sfs must be TRUE
  if(seq==FALSE && sfs ==FALSE) {
    print("no output produced")
  }
  
  if(seq==TRUE && sfs==TRUE)
    layout(matrix(c(1,2), ncol=2, nrow=1))
  else 
    layout(matrix(c(1), ncol=1, nrow=1))

  # wait<0 means: ask for input after a plot
  if(wait<0) {
    par(ask=T)
  } else {
    par(ask=FALSE)
  }

  # make starting configuration
  S<-4*N*u*sum(1/(1:(N-1)))
  A<-createNeutralA(N,S,u)

  # First, plot everything without beneficial SNP
  if(seq) {
    plot(c(0,1), c(1,N), type='n', axes=FALSE, xlab="", ylab="")
    plotA(A)
  }

  # Create beneficial SNP
  isB<-matrix(0, ncol=1, nrow=N)
  i<-as.integer(N*runif(1)+1)
  isB[i,1]<-1
  isB<-list(isB, c(0.5), N)

  if(sfs) {
    plotSFS(A)
  }

  # plot with beneficial allele
  if(seq) {
    plot(c(0,1), c(1,N), type='n', axes=FALSE, xlab="", ylab="")
    plotA(A)
    plotA(isB, col="red")
  }
  if(sfs) plotSFS(A)

  while(sum(isB[[1]][,1])!=N) {
    p<-runif(1)
    if(p < 2*rho*N/(2*(rho+u)*N+N-1)) {
      # recombination
      i<-1+as.integer(N*runif(1))
      j<-1+as.integer(N*runif(1))
      brea<-runif(1)
      A<-recombination(A, i, j, brea)
      isB<-recombination(isB, i, j, brea)
    } else if(p < 2*(rho+u)*N/(2*(rho+u)*N+N-1)) {
      # mutation
      i<-1+as.integer(N*runif(1))
      site<-runif(1)
      A<-mutation(A, i, site)
    } else {
      # resampling
      i<-sum(isB[[1]][,1])
      j<-trans(N,s,i)
      if(i==j) {
        k<-1+as.integer(N*runif(1))
        l<-sample(1:N, 1, prob=1-isB[[1]][k]+isB[[1]])
      } else if(j<i) {
        k<-sample(1:N, 1, prob=1-isB[[1]])
        l<-sample(1:N, 1, prob=isB[[1]])
      } else if(j>i) {
        k<-sample(1:N, 1, prob=isB[[1]])
        l<-sample(1:N, 1, prob=1-isB[[1]])
      }
      A<-resampling(A, k, l)
      isB<-resampling(isB, k, l)
      }
      if(wait>0) Sys.sleep(wait)

    if(showFixed==FALSE) A<-deleteFixed(A)
    if(seq) {
      plot(c(0,1), c(1,N), type='n', axes=FALSE, xlab="", ylab="")
      plotA(A)
      plotA(isB, col="red")
    }
    if(sfs) 
      plotSFS(A)
  }
}

