\name{popSubdivision}

\alias{popSubdivision}

\alias{plot.popSubdivision}

\title{Substructured Populations}
\description{Show the effect of substructured populations on genetic drift}

\usage{
popSubdivision( N=50, demes=10, init.A=0.5, m=0, 
                mainland=FALSE, stoptime=500 )

\method{plot}{popSubdivision}( x, what=c( "deme.freq.A" ), col=list(), 
      xlim=NULL, ylim=c( 0, 1 ), xlab="time", ylab="freq",
      legend.pos="bottomright", ... )
}

\arguments{
   \item{x}{object of class \code{popSubdivision}}
   \item{N}{population size of each deme, the demes have the same size}
   \item{demes}{number of demes}
   \item{init.A}{initial frequency of allele A}
   \item{m}{migration rate}
   \item{mainland}{logical; if it is TRUE migration comes from a mainland population that has a fixed allele frequency, this may sound unrealistic, but this is the original island model by Wright, or at least this ist wehn Fst equals 1/(1-4Nm). If it is FALSE then there is no mainland but the allele frequency in the migrantpool is the mean allele frequency over all demes. }
   \item{stoptime}{stoptime i.e. the number of generations to simulate}
   \item{what}{which curves to show (possible: "deme.freq.A", "migrand.freq.A" )}
   \item{col}{list with colors for each curve defined in \code{what}}
   \item{xlim}{range of x axis; default: 0 to stoptime}
   \item{ylim}{range of y axis; default: 0 to 1}
   \item{xlab}{annotation of x axis}
   \item{ylab}{annotation of y axis}
   \item{legend.pos}{position of legend. See Details}
   \item{\dots}{additional arguments}
}

\details{
This function shows the effect of pupulation subdivision. It can be used to illustrate (1) the Wahlund effect, (2) the effect that drift iscounterbalanced by migration and (3) the formula 1/(1+4Nm). 
The number of migrants are random numbers drawn from a binomial distribution. They differ from deme to deme. The number of migrants with allele A as well as the number of residents with allele A is random and different by deme.

The location of the legend can be specified by setting 'legend.pos' to a single keyword from the list '"bottomright"', '"bottom"', '"bottomleft"', '"left"', '"topleft"', '"top"', '"topright"', '"right"' and '"center"'. This places the legend on the inside of the plot frame at the given location.
}

\value{
An object of class \code{popSubdivision}, a list with elements
\item{deme.freq.A}{a matrix of the allele frequencies of allele A per generation per deme}
\item{migrant.freq.A}{a vector of the allele frequencies of allele A in the migrant pool per generation}
\item{Hdeme}{a matrix of heterozygosity per generation per deme}
\item{Hs}{a vector of the mean heterozygosity over all demes per generation}
\item{Ht}{a vector of the expected hereozygosity if the population wre panmictic per generation}
\item{Fst}{a vector of the deficit of hererozygots due to population subdivision per generation}
\item{F.predicted}{the Fst that is predicted by the formula Fst=1/(1+4Nm)}
\item{fixed}{a matrix of logical values indicating whether an allele has come to fixation per deme per generation}
\item{prop.fixed}{a vector of the proportion of demes in which an allele has come to fixation per generation}
}

\references{\url{http://www.zi.biologie.uni-muenchen.de/~pfaffelhuber/teaching/popgenCourse.html}}
\author{Ulrike Feldmann}

\examples{
res <- popSubdivision()

plot( res )
plot( res, what="migrant.freq.A" )
plot( res, what=c( "deme.freq.A", "migrant.freq.A" ) )
plot( res, col=list( "deme.freq.A" = colours()[1:10] ) )

res <- popSubdivision( m=0.2 )
res <- popSubdivision( m=0.1, mainland=TRUE )
res <- popSubdivision( m=0.1, mainland=TRUE, init.A = 0.15 )
}

\keyword{distribution}
